unit g_Gun_mini;

interface
uses
  Windows,Classes,
  HGE,HGESprite,HGEParticle,
  g_Types,g_cfg,g_math,g_Options,g_Utils;

type
  TBullet= class
    X,Y : Single;
    Angle : single;
    particle : IHGEParticleSystem;
    active : boolean;
  end;

  TGunMini = class
   public
     X,Y : single;
     inShot : boolean;
     Damage_Time  : single;
     Damage_Delay : single;
     Tex : ITexture;
     Spr : IHGESprite;
     TexParticle : ITexture;
     List : TList;
     FireDemage : word;                                                       //  
     ReloadTime : single;                                                       //  
     ReloadDelay : single;                                                      //   
     BulletSpeed : single;                                                      //  
     Color : Cardinal;
     sndFire : array [1..4]of IEffect;
   public
    constructor Create;
    destructor Destroy;
    procedure Init;
    procedure Shot;
    procedure Render();
    procedure Update(const delta,_X,_Y : single);
    procedure AddBullet(const _X,_Y,_Angle : single);
    procedure Dead(id : word);
    procedure MiniBoom(id : word);
  end;

implementation
  uses
    g_Player,g_monster_fly,g_monster_eye,g_monster_nlo,g_monster_bee,
    g_boss_BigHead;

constructor TGunMini.Create;
var
  FileName : string;
  i : byte;
begin
  inherited;
  FileName := 'data\cfg\gun_mini.cfg';
  Tex := _HGE.Texture_Load(Cfg_ReadStr(FileName,'TEXTURE_BULLET','bullet.png'));
  Spr := THGESprite.Create(Tex,0,0,Tex.GetWidth,Tex.GetHeight);
  Spr.SetHotSpot(Tex.GetWidth / 2,Tex.GetHeight / 2);
  Color := $ff55ccff;
  Spr.SetColor(Color);
  TexParticle := _HGE.Texture_Load(Cfg_ReadStr(FileName,'TEXTURE_PARTICLE_BALL','particles.png'));
  List := TList.Create;
  BulletSpeed := Cfg_ReadInt(FileName,'BULLET_SPEED',6)*WndAspect;
  FireDemage := Cfg_ReadInt(FileName,'FIRE_DAMAGE',5);
  ReloadDelay := Cfg_ReadInt(FileName,'RELOAD_DELAY',10)/100;
  for I := 1 to 4 do
    sndFire[i] := _HGE.Effect_Load(Cfg_ReadStr('data\cfg\gun_mini.cfg','SOUND_FIRE_'+inttostr(i),'mini_fire.wav'));
end;

destructor TGunMini.Destroy;
begin
  sndFire[1] := nil;
  sndFire[2] := nil;
  sndFire[3] := nil;
  sndFire[4] := nil;
  inherited;
end;

procedure TGunMini.Init;
begin
  inShot := false;
  //sndFire := SoundEngine.LoadSound(dir_local+Cfg_ReadStr(FileName,'sndFire','data\snd\default.wav'));
  List.Clear;
end;

procedure TGunMini.Shot;
begin
  case Player.level of
    1 : begin
      AddBullet(X,Y,270);
      sndFire[1].PlayEx(Options.SoundVolume);
    end;
    2 : begin
      AddBullet(X-20*WndAspect,Y,270);
      AddBullet(X+20*WndAspect,Y,270);
      sndFire[2].PlayEx(Options.SoundVolume);
    end;
    3 : begin
      AddBullet(X,Y,270);
      AddBullet(X-20*WndAspect,Y,270);
      AddBullet(X+20*WndAspect,Y,270);
      sndFire[3].PlayEx(Options.SoundVolume);
    end;
    4..99 : begin
      AddBullet(X,Y,270);
      AddBullet(X-20*WndAspect,Y,270);
      AddBullet(X+20*WndAspect,Y,270);
      AddBullet(X-45*WndAspect,Y,240);
      AddBullet(X+45*WndAspect,Y,300);
      sndFire[4].PlayEx(Options.SoundVolume);
    end;
  end;
end;

procedure TGunMini.AddBullet(const _X,_Y,_Angle : single);
var
 id : integer;
 i : word;
begin
  id := -1;
  i:=0;
  while i<List.Count do begin
    with TBullet(List.Items[i]) do begin
      if not active then begin
        ID := i;
        break;
      end;
    end;
    inc(i);
  end;

  if id = -1 then id := List.Add(TBullet.Create);
  with TBullet(List.Items[id]) do begin
    X := _X;
    Y := _Y;
    Angle := _Angle;
    active := true;
  end;
end;

procedure TGunMini.Dead(id : word);
begin
  TBullet(List.Items[id]).active := false;
end;

procedure TGunMini.MiniBoom(id : word);
begin
  ToggleParSys(TBullet(List.Items[id]).Particle,'data\particles\particle_gun_mini.psi',TexParticle,64,0,32,
               round(TBullet(List.Items[id]).X),round(TBullet(List.Items[id]).Y),
               BLEND_ALPHABLEND or BLEND_COLORMUL or BLEND_ALPHAADD,$ff00ffff,Color);
  Dead(id);
end;

procedure TGunMini.Update(const delta,_X,_Y : single);
var i,j : word;
begin
  X := _X;
  Y := _Y;
  ReloadTime := ReloadTime + delta;
  if (inShot)and(ReloadTime > ReloadDelay) then begin
    Shot;
    inShot := false;
    ReloadTime := 0;
  end else inShot := false;

  i:=0;
  while i<List.Count do begin
    if TBullet(List.Items[i]).active then
    with TBullet(List.Items[i]) do begin
      X := X + BulletSpeed*cos(deg2rad*Angle);
      Y := Y + BulletSpeed*sin(deg2rad*Angle);
      if Y < 0 then dead(i) else begin
        j:=0;
        while j<Monsters_FLY.List.Count do begin
          if TMonster_FLY(Monsters_FLY.List.Items[j]).Life > 0 then
          if Distance2D(X,Y,TMonster_FLY(Monsters_FLY.List.Items[j]).X,TMonster_FLY(Monsters_FLY.List.Items[j]).Y)<Monsters_FLY.Radius then begin
            Monsters_FLY.Damage(j,FireDemage);
            MiniBoom(i);
            break;
          end;
          inc(j);
        end;
        if active then begin
          j:=0;
          while j<Monsters_EYE.List.Count do begin
            if TMonster(Monsters_EYE.List.Items[j]).Life > 0 then
            if Distance2D(X,Y,TMonster(Monsters_EYE.List.Items[j]).X,TMonster(Monsters_EYE.List.Items[j]).Y)<Monsters_EYE.Radius then begin
              Monsters_EYE.Damage(j,FireDemage);
              MiniBoom(i);
              break;
            end;
            inc(j);
          end;
        end;
        if active then begin
          j:=0;
          while j<Monsters_NLO.List.Count do begin
            if TMonster(Monsters_NLO.List.Items[j]).Life > 0 then
            if Distance2D(X,Y,TMonster(Monsters_NLO.List.Items[j]).X,TMonster(Monsters_NLO.List.Items[j]).Y)<Monsters_NLO.Radius then begin
              Monsters_NLO.Damage(j,FireDemage);
              MiniBoom(i);
              break;
            end;
            inc(j);
          end;
        end;
        if active then begin
          j:=0;
          while j<Monsters_BEE.List.Count do begin
            if TMonster_BEE(Monsters_BEE.List.Items[j]).Life > 0 then
            if Distance2D(X,Y,TMonster_BEE(Monsters_BEE.List.Items[j]).X,TMonster_BEE(Monsters_BEE.List.Items[j]).Y)<Monsters_BEE.Radius then begin
              Monsters_BEE.Damage(j,FireDemage);
              MiniBoom(i);
              break;
            end;
            inc(j);
          end;
        end;
        if active then begin
          if Boss_BigHead.Life > 0 then
          if Distance2D(X,Y,Boss_BigHead.X,Boss_BigHead.Y)<Boss_BigHead.Radius then begin
            Boss_BigHead.Damage(FireDemage);
            MiniBoom(i);
            break;
          end;
        end;
        if Boss_BigHead.Life > 0 then begin
          if active then
          if Boss_BigHead.Hand[0].Life > 0 then
          if Distance2D(X,Y,Boss_BigHead.Hand[0].X,Boss_BigHead.Hand[0].Y)<Boss_BigHead.Hand[0].Radius then begin
            Boss_BigHead.Hand[0].Damage(FireDemage,Boss_BigHead.SndDead);
            MiniBoom(i);
            break;
          end;
          if active then
          if Boss_BigHead.Life > 0 then
          if Boss_BigHead.Hand[1].Life > 0 then
          if Distance2D(X,Y,Boss_BigHead.Hand[1].X,Boss_BigHead.Hand[1].Y)<Boss_BigHead.Hand[1].Radius then begin
            Boss_BigHead.Hand[1].Damage(FireDemage,Boss_BigHead.SndDead);
            MiniBoom(i);
            break;
          end;
        end;
      end;
    end;
    inc(i);
  end;
end;

procedure TGunMini.Render();
var i : word;
begin
  i := 0;
  while i < List.Count do begin
    if TBullet(List.Items[i]).active then
    with TBullet(List.Items[i]) do
      Spr.RenderEx(X,Y,deg2rad*Angle,1*WndAspect);
  inc(i);
  end;
end;

end.
