unit g_LevelManager;

interface
uses
  HGE,
  g_Utils,g_cfg;

type
  TLevelManager = class
  public
    LevelCount,LevelCurrent : byte;
    Levels : array of string;
    LevelTime,LevelDelay : single;
    Day,Hour,Minute,Second : byte;
    PrepareTime,PrepareDelay : single;                                         //      
    isPlaying : boolean;
    Finish : boolean;
    Winner : boolean;
  public
    constructor Create;
    destructor Destroy;
    procedure NewGame;
    procedure Update(const delta : single);
    procedure LevelEnd;
    procedure NextLevel;
    procedure GameOver(_Winner : boolean=false);
  end;

implementation
uses
  g_Game,g_Types,g_Camera,g_top,g_Options,g_Player;

constructor TLevelManager.Create;
var
  f : TextFile;
begin
  inherited Create;
  Finish := false;
  PrepareDelay := Cfg_ReadInt('data\cfg\map.cfg','LEVEL_CAMERA_MOVE_DELAY',2);
  World_Create();
  AssignFile(f,'data\cfg\level.cfg');
  Reset(f);
  LevelCount := 0;
  while not eof(f) do begin
    inc(LevelCount);
    SetLength(Levels,LevelCount);
    ReadLn(f,Levels[LevelCount-1]);
  end;
  CloseFile(f);
end;

destructor TLevelManager.Destroy;
begin

  inherited Destroy;
end;

procedure TLevelManager.NewGame;
begin
  Game.Channel := Game.Music.Play(true,Options.MusicVolume);
  World_Init();
  LevelTime := 0;
  PrepareTime := 0;
  LevelCurrent := 0;
  Map.Load(levels[LevelCurrent]);
  LevelTime := 0;
  isPlaying := true;
  Game.Pause := false;
  Finish := false;
  Winner := false;
end;

procedure TLevelManager.NextLevel;
begin
  LevelTime := 0;
  PrepareTime := 0;
  inc(LevelCurrent);
  ParticleManager.KillAll;
  Player.Init;
  Map.Load(levels[LevelCurrent]);
  LevelTime := 0;
  isPlaying := true;
  Game.Pause := false;
  Finish := false;
end;

procedure TLevelManager.LevelEnd;
begin
  Game.Pause := true;
  Finish := true;
  isPlaying := false;
  if LevelCurrent < LevelCount-1 then NextLevel else GameOver;
end;

procedure TLevelManager.GameOver(_Winner : boolean=false);
begin
  TOP.Check(Profile.PlayerName,Game.GamePlayColorAll[random(7)],0);
  Top.SaveTOP;
  Finish := true;
  Winner := _Winner;
  isPlaying := false;
end;

procedure TLevelManager.Update(const delta : single);
begin
  ParticleManager.Update(delta);
  if not Game.Pause then
  if isPlaying then begin
    {LevelTime := LevelTime + delta;
    if LevelTime > PrepareDelay then begin
    case _Type of
      ltTime:  LevelEnd;
      ltWiner: begin
        PlayerManager.LoserDead;
        LevelTime := 0;
        inc(LoserCount);
        if (TPlayer(PlayerManager.List.Items[PlayerManager.Players[0]])._State = psDead) then
         if not Game.TwoPlayer then LevelEnd else
           if (TPlayer(PlayerManager.List.Items[PlayerManager.Players[1]])._State = psDead) then
             LevelEnd;
        if PlayerManager.List.Count=LoserCount+1 then LevelEnd;
      end;
    end;  }
      //TimeToNormalTime(round(LevelDelay-LevelTime),day,hour,minute,second);
      World_Update(delta);
    //end;
  end;
end;

end.
