unit g_bullet_manager;

interface
uses
  Windows,
  HGE,HGESprite,HGEParticle,Classes,
  g_Types,g_cfg,g_math;

type
  TBulletType=(btMini,btRocket,btUnknown);
  TBullet= class
    X,Y : Single;
    Angle : single;
    _Type : TBulletType;
    DamageDist : byte;
    Speed : single;                                                            //  
    active : boolean;
    constructor Cerate;
  end;

  TBulletManager = class
    Tex : ITexture;
    Spr : IHGESprite;
    TexParticle : ITexture;
    particle : IHGEParticleSystem;
    PosX,PosY : single;
    List : TList;
    StandartBullet : array [0..3] of TBullet;
    BulletName : array [0..3] of String[6];
    FireDemage : single;                                                       //  
    ReloadTime : single;                                                       //  
    ReloadDelay : single;                                                      //   
    constructor Create(FileName : string);
    procedure Init;
    procedure AddBullet(__Type : TBulletType;const _X,_Y,_Angle : single);
    procedure Dead(id : word);
    procedure Update;
    procedure Render;

    procedure MiniBoom(id : word);
    procedure RocketBoom(id : word);
  end;

var
  BulletManager : TBulletManager;

implementation
uses
  g_Player,g_monster_fly,g_monster_eye,g_monster_nlo;

constructor TBullet.Cerate;
begin
  inherited;

end;

constructor TBulletManager.Create(FileName : string);
var
  I: Integer;
begin
  inherited Create;
  Tex := _HGE.Texture_Load(Cfg_ReadStr(FileName,'TEXTURE_BULLET','bullet.png'));
  Spr := THGESprite.Create(Tex,0,0,Tex.GetWidth,Tex.GetHeight);
  Spr.SetHotSpot(Tex.GetWidth / 2,Tex.GetHeight / 2);
  Spr.SetColor($ffffffff);
  TexParticle := _HGE.Texture_Load(Cfg_ReadStr(FileName,'TEXTURE_PARTICLE_BALL','particles.png'));
  List := TList.Create;
  BulletName[0] := 'MINI';
  BulletName[1] := 'ROCKET';
  BulletName[2] := 'VOICE';
  BulletName[3] := 'VOICE';
  for I := 0 to 3 do begin
    StandartBullet[i] := TBullet.Create;
    StandartBullet[i]._Type := TBulletType(i);
    with StandartBullet[i] do begin
        Angle :=90;
        Speed := Cfg_ReadInt(FileName,BulletName[i]+'_BULLET_SPEED',6)*WndAspect;
        {FireDemage := Cfg_ReadInt(FileName,BulletName[i]+'_FIRE_DAMAGE',5);
        ReloadDelay := Cfg_ReadInt(FileName,BulletName[i]+'_MINI_RELOAD_DELAY',10)/100; }
        ReloadDelay := Cfg_ReadInt(FileName,BulletName[i]+'_RELOAD_DELAY',10)/100;
    end;
  end;
end;

procedure TBulletManager.Init;
begin
  List.Clear;
end;

procedure TBulletManager.AddBullet(__Type : TBulletType;const _X,_Y,_Angle : single);
var
 id : integer;
 i : word;
begin
  id := -1;
  i:=0;
  while i<List.Count do begin
    with TBullet(List.Items[i]) do begin
      if not active then begin
        ID := i;
        break;
      end;
    end;
    inc(i);
  end;

  if id = -1 then id := List.Add(TBullet.Create);
  with TBullet(List.Items[id]) do begin
    _Type := __Type;
    X := _X;
    Y := _Y;
    Angle := _Angle;
    Speed := StandartBullet[ord(_Type)].Speed;
  end;
end;

procedure TBulletManager.Dead(id : word);
begin
  TBullet(List.Items[id]).active := false;
end;

procedure TBulletManager.Update;
var i,j : word;
begin
  i:=0;
  while i<List.Count do begin
    with TBullet(List.Items[i]) do begin
      X := X + Speed*cos(deg2rad*Angle);
      Y := Y + Speed*sin(deg2rad*Angle);
      if (Y<0)or(Y>WND_HEIGHT)or(X<0)or(X>WND_WIDTH) then Dead(i) else begin
        X := X + Speed*cos(deg2rad*Angle);
        Y := Y + Speed*sin(deg2rad*Angle);
        j:=0;
        while j<Monsters_FLY.List.Count do begin
          if TMonster(Monsters_FLY.List.Items[j]).Life > 0 then
          if Distance2D(X,Y,TMonster(Monsters_FLY.List.Items[j]).X,TMonster(Monsters_FLY.List.Items[j]).Y)< Monsters_FLY.Radius then begin
            case _Type of
              btMini:   MiniBoom(i);
              btRocket: RocketBoom(i);
              btUnknown: ;
            end;
            break;
          end;
          inc(j);
        end;
        if active then begin
          j:=0;
          while j<Monsters_EYE.List.Count do begin
            if TMonster(Monsters_EYE.List.Items[j]).Life > 0 then
            if Distance2D(X,Y,TMonster(Monsters_EYE.List.Items[j]).X,TMonster(Monsters_EYE.List.Items[j]).Y)<Monsters_EYE.Radius then begin
            case _Type of
              btMini:   MiniBoom(i);
              btRocket: RocketBoom(i);
              btUnknown: ;
            end;
            break;
            end;
            inc(j);
          end;
        end;
        if active then begin
          j:=0;
          while j<Monsters_NLO.List.Count do begin
            if TMonster(Monsters_NLO.List.Items[j]).Life > 0 then
            if Distance2D(X,Y,TMonster(Monsters_NLO.List.Items[j]).X,TMonster(Monsters_NLO.List.Items[j]).Y)<Monsters_NLO.Radius then begin
              case _Type of
                btMini:   MiniBoom(i);
                btRocket: RocketBoom(i);
                btUnknown: ;
              end;
              break;
            end;
            inc(j);
          end;
        end;
      end;
    end;
    inc(i);
  end;
end;

procedure TBulletManager.Render;
var i : word;
begin
  i := 0;
  while i < List.Count do begin
    with TBullet(List.Items[i]) do begin
      Spr.RenderEx(X,Y,deg2rad*Angle,1*WndAspect);
    end;
  inc(i);
  end;
end;


procedure TBulletManager.MiniBoom(id : word);
begin
  ToggleParSys(Particle,'data\particles\particle_collision.psi',TexParticle,64,0,32,
               round(TBullet(List.Items[id]).X),round(TBullet(List.Items[id]).Y),
               BLEND_ALPHABLEND or BLEND_COLORMUL or BLEND_ALPHAADD,$ffff8800,
               $ffffffff);
  Dead(id);
end;

procedure TBulletManager.RocketBoom(id : word);
begin
  //        
  //    DamageDist
  //     
  ToggleParSys(Particle,'data\particles\particle_collision.psi',TexParticle,64,64,32,
               round(TBullet(List.Items[id]).X),round(TBullet(List.Items[id]).Y),
               BLEND_ALPHABLEND or BLEND_COLORMUL or BLEND_ALPHAADD,$ffffff00,
               $ffff0000);
  Dead(id);
end;

end.
