unit g_Map;

interface
uses
  Classes,
  HGE,HGESprite,HGEParticle,
  g_math,g_cfg,g_utils;

type
  TStar = class
    X,Y,Z,
    Speed,Angle,
    size : single;
    Mode : byte;
  end;

  TStarManager = class
    List : TList;
    Tex : ITexture;
    Spr : IHGESprite;
    constructor Create;
    procedure Init;
    procedure Update(delta : single);
    procedure Render;
    procedure StarReload(ID : word);
  end;

  TNebula = class
    List : TList;
    Tex : ITexture;
    Spr : IHGESprite;
    active : boolean;
    CreateDelay,CreateTime : single;
    Color : Cardinal;
    constructor Create;
    procedure Init;
    procedure Update(delta : single);
    procedure Render;
    procedure Generate(const _X,_Y,_Radius,_Speed : single;_Color : Cardinal);
  end;

  TTurn = class
    X : single;
    Time : word;
    Mode : byte;
  end;

  TMap = class
    public
      Name : string;                                    //  
      Description : PChar;                              // 
      levelName : string[12];
      List : TList;
      MapTime : single;

      StarManager : TStarManager;
      Nebula : TNebula;
      constructor Create;
      procedure Load(FileName : string);
      procedure Render;
      procedure UpLayerRender;
      procedure Update(delta : single);
  end;

implementation
uses
  g_Types,g_Game,g_gui,g_monster_fly,g_monster_eye,g_monster_nlo,g_monster_bee,
  g_boss_BigHead;

constructor TStarManager.Create;
begin
  inherited;
  Tex := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\map.cfg','TEXTURE_STAR','data\graphics\star.png'));
  Spr := THGESprite.Create(Tex,0,0,Tex.GetWidth,Tex.GetHeight);
  Spr.SetHotSpot(Tex.GetWidth/2,Tex.GetHeight/2);
  Spr.SetColor($ffffffff);
  List := TList.Create;
end;

procedure TStarManager.Init;
var i,ID : word;
begin
  List.Clear;
  for I := 0 to 60 do begin
    ID := List.Add(TStar.Create);
    with TStar(List.Items[ID]) do begin
      StarReload(id);
      Y := random(WND_HEIGHT);
    end;
  end;
end;

procedure TStarManager.Update;
var i : word;
begin
  //Spr.RenderStretch(0,0,800*WndAspect,600*WndAspect);
  for I := 0 to List.Count-1 do begin
    with TStar(List.Items[i]) do begin
      X := X + Speed*cos(deg2rad*Angle);
      Y := Y + Speed*sin(deg2rad*Angle);
      if Y > WND_HEIGHT then begin
        StarReload(i);
      end;
    end;
  end;
end;

procedure TStarManager.Render;
var i : word;
begin
  _HGE.Gfx_Clear(0);
   for I := 0 to List.Count-1 do
    with TStar(List.Items[i]) do begin
      Spr.RenderEx(X,Y,deg2rad*Angle,size*WndAspect);
    end;
end;

procedure TStarManager.StarReload(ID : word);
begin
  with TStar(List.Items[iD]) do begin
    X := random(WND_WIDTH);
    Y := -random(100)*WndAspect;
    Angle := 80+random(20);
    Speed := (5+random(200))/100*WndAspect;
    size := (50+random(250))/1000;
  end;
end;

constructor TNebula.Create;
begin
  inherited;
  Tex := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\map.cfg','TEXTURE_STAR','data\graphics\star.png'));
  Spr := THGESprite.Create(Tex,0,0,Tex.GetWidth,Tex.GetHeight);
  Spr.SetHotSpot(Tex.GetWidth/2,Tex.GetHeight/2);
  Spr.SetColor($ffffffff);
  List := TList.Create;
end;

procedure TNebula.Init;
begin
  List.Clear;
  CreateDelay := 1;
  Generate(random(WND_WIDTH),random(WND_HEIGHT),3+random(40)/10,0.2+Random(30)/100,Game.GameplayColorALL[Random(Game.GameplayColorCount)]);
end;

procedure TNebula.Update;
var i : word;
begin
  if not active then begin
    CreateTime := CreateTime+delta;
    if CreateTime>CreateDelay then begin
      Generate(random(WND_WIDTH),-random(SCR_CENTER_H),3+random(40)/10,0.2+Random(30)/100,Game.GameplayColorALL[Random(Game.GameplayColorCount)]);
      CreateTime := 0;
      CreateDelay := 3;
    end;
  end else begin
    active := false;
    for I := 0 to List.Count-1 do begin
      with TStar(List.Items[i]) do begin
        X := X + Speed*cos(deg2rad*Angle);
        Y := Y + Speed*sin(deg2rad*Angle);
        if Y < WND_HEIGHT then begin
          active := true;
        end;
      end;
    end;
  end;
end;

procedure TNebula.Render;
var i : word;
begin
  _HGE.Gfx_Clear(0);
   for I := 0 to List.Count-1 do
    with TStar(List.Items[i]) do begin
      Spr.SetColor(Color);
      Spr.RenderEx(X,Y,0,size*WndAspect);
    end;
end;

procedure TNebula.Generate(const _X,_Y,_Radius,_Speed : single;_Color : Cardinal);
var
  i,j,ID : word;
  ang,siz,rad : single;
begin
  Color := _Color;
  rad := 0;
  ang := 0;
  siz := 0.20+random(150)/1000;
  for j := 0 to 15 do begin
    rad := rad + _Radius;
    ang := ang+360/16;
    siz := siz - 0.02;
    for I := 0 to 19 do begin
      ID := List.Add(TStar.Create);
      with TStar(List.Items[ID]) do begin
        X := _X+(Rad+random(round(Rad/2*10))/10)*cos(deg2rad*Ang+i*18);
        Y := _Y+(Rad+random(round(Rad/2*10))/10)*sin(deg2rad*Ang+i*18);
        Angle := 90;
        Speed := _Speed*WndAspect;
        size := siz+random(100)/1000;
      end;
    end;
  end;
  active := true;
end;

constructor TMap.Create;
begin
  inherited;
  List := TList.Create;
  StarManager := TStarManager.Create;
  Nebula := TNebula.Create;
end;

procedure TMap.Load(FileName : string);
var
  f : file of byte;
  Count,i,ID : word;
  _X,_Time,_Mode : byte;
begin
  // Clear
  List.Clear;
  MapTime := 0;
  if not FileExists(FileName) then begin
     _HGE.System_Log(FileName + ' - file not found');
    exit;
  end;
  AssignFile(f,FileName);
  Reset(f);
  BlockRead(f,LevelName,12);
  BlockRead(f,Count,2);
  for I := 0 to Count - 1 do begin
    ID := List.Add(TTurn.Create);
    BlockRead(F,_X,1);
    BlockRead(F,_Time,1);
    BlockRead(F,_Mode,1);
    TTurn(List.Items[ID]).X := MONSTER_RADIUS*WndAspect+_X*MONSTER_RADIUS*2*WndAspect;
    TTurn(List.Items[ID]).Time := _Time;
    TTurn(List.Items[ID]).mode := _Mode;
  end;

  CloseFile(f);
  StarManager.Init;
  Nebula.Init;
  GUI.HintLevelName.Add(SCR_CENTER_W,SCR_CENTER_H,levelName,Game.GameplayColorAll[Random(Game.GameplayColorCount)]);
end;

procedure TMap.Render;
begin
  //Spr.RenderStretch(0,0,800*WndAspect,600*WndAspect);
  _HGE.Gfx_Clear(0);
   StarManager.Render;
   Nebula.Render;
end;

procedure TMap.UpLayerRender;
begin
  //Spr2.RenderStretch(0,0,800,600);
end;

procedure TMap.Update(delta : single);
var
  I: Integer;
begin
  MapTime := MapTime + delta;
  I := 0;
  while i<List.Count do begin
      if MapTime>TTurn(List.Items[i]).Time then begin
        case TTurn(List.Items[i]).Mode of
          1 : Monsters_FLY.AddMonster(TTurn(List.Items[i]).X,10);
          2 : Monsters_EYE.AddMonster(TTurn(List.Items[i]).X,10);
          3 : Monsters_NLO.AddMonster(TTurn(List.Items[i]).X,10);
          4 : Monsters_BEE.AddMonster(TTurn(List.Items[i]).X,10);
          5 : Boss_BigHead.Go(TTurn(List.Items[i]).X,10);
        end;
        List.Delete(i);
        continue;
      end;
    inc(i);
  end;
  //if (List.Count=0)then LevelManager.GameOver;
  StarManager.Update(delta);
  Nebula.Update(delta);
end;

end.
