unit g_Menu;

interface
uses
  HGE,HGESprite,HGEFont,HGEGUI,Menuitem,HGEAnim,
  g_Types,g_Options,g_cfg,g_about,g_Top,g_Utils,g_language,g_LevelManager,g_Math;

type
  TMenuButton = class
    Tex : ITexture;
    Anim : IHGEAnimation;
    isActive : boolean;
    size : single;
    X,Y : single;
    StartSelect : boolean;
    constructor Create(_Type : string);
    procedure Select(mouseX,mouseY : single);
    procedure Update;
    procedure Draw;
  end;

  TMenu = class
    logo_fnt : IHGEFont;
    Fnt : IHGEFont;
    Tex,CursorTex,
    ProfileEnterTex,ProfileEnterMuchTex : ITexture;
    Spr,CursorSpr,ProfileEnterUPSpr,ProfileEnterDownSpr,
    ProfileEnterMuchSpr1 : IHGESprite;
    SndClick: IEffect;
    GUI: IHGEGUI;
    LastId: Integer;
    Music : IStream;
    Channel : IChannel;
    ItemsCount : byte;
  public
    constructor Create;
    procedure Activate;
    procedure DeActivate;
    procedure Update(const delta : single);
    procedure Draw;
    destructor Destroy;
  end;

var
  Menu : TMenu;

implementation
uses
  g_Game;

constructor TMenuButton.Create(_Type : string);
begin
  inherited Create;
  Tex := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\menu.cfg',_Type,'menu.png'));
  Anim := THGEAnimation.Create(Tex,2,0.0,0,0,Tex.GetHeight,Tex.GetHeight);
  Anim.SetHotSpot(Anim.GetWidth/2,Anim.GetHeight/2);
  isActive := false;
  size := 0.9;
  StartSelect := false;
end;

procedure TMenuButton.Select(mouseX,mouseY : single);
begin
  if (mouseX>X - 32*WndAspect)and(mouseX<X + 32*WndAspect)and(mouseY>Y - 32*WndAspect)and(mouseY<Y + 32*WndAspect) then begin
    if not isActive then StartSelect := true else StartSelect := false;
    isActive := true;
    Aspire(size,1,1*_HGE.Timer_GetDelta);
  end else begin
    isActive := false;
    Aspire(size,0.9,1*_HGE.Timer_GetDelta);
  end;
end;

procedure TMenuButton.Draw;
begin
  Anim.RenderEx(X,Y,0,Size*WndAspect,Size*WndAspect);
end;

procedure TMenuButton.Update;
begin

end;

constructor TMenu.Create;
begin
  inherited;
  WndAspect := WND_WIDTH/800;
  Fnt := THGEFont.Create(Cfg_ReadStr('data\cfg\menu.cfg','FONT','data\fonts\default.fnt'));
  SndClick := _HGE.Effect_Load(Cfg_ReadStr('data\cfg\menu.cfg','SOUND_CLICK','data\sounds\click.wav'));
  Music := _HGE.Stream_Load(Cfg_ReadStr('data\cfg\menu.cfg','MUSIC','data\music\menu_background.ogg'));
  Channel := Music.Play(true,Options.MusicVolume);
  Tex := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\menu.cfg','TEXTURE','menu.png'));
  Spr := THGESprite.Create(Tex,0,0,Tex.GetWidth,Tex.GetHeight);
  //Spr.SetHotSpot(Tex.GetWidth div 2,Tex.GetHeight div 2);
  CursorTex := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\menu.cfg','TEXTURE_CURSOR','corsor.png'));
  CursorSpr := THGESprite.Create(CursorTex,0,0,CursorTex.GetWidth,CursorTex.GetHeight);
  CursorSpr.SetHotSpot(0,0);
  ProfileEnterTex := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\menu.cfg','TEXTURE_PROFILE_ENTER','ProfileEnter_Table.png'));
  ProfileEnterUpSpr := THGESprite.Create(ProfileEnterTex,0,0,ProfileEnterTex.GetWidth,32);
  ProfileEnterUpSpr.SetHotSpot(ProfileEnterUpSpr.GetWidth/2,0);
  ProfileEnterDownSpr := THGESprite.Create(ProfileEnterTex,0,32,ProfileEnterTex.GetWidth,32);
  ProfileEnterDownSpr.SetHotSpot(ProfileEnterDownSpr.GetWidth/2,0);
  ProfileEnterMuchTex := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\menu.cfg','TEXTURE_PROFILE_ENTER_1_IMG','ProfileEnter_Table.png'));
  ProfileEnterMuchSpr1 := THGESprite.Create(ProfileEnterMuchTex,0,0,ProfileEnterMuchTex.GetWidth,ProfileEnterMuchTex.GetHeight);// div 2);
  ProfileEnterMuchSpr1.SetHotSpot(ProfileEnterMuchSpr1.GetWidth/2,ProfileEnterMuchSpr1.GetHeight/2);
  Top := TTop.Create;

  GUI := THGEGUI.Create;
  ItemsCount := 6;
  GUI.AddCtrl(THGEGUIMenuItem.Create(1,Fnt,SndClick,400*WndAspect,260*WndAspect,0.0,Language.Menu.Continue));
  GUI.AddCtrl(THGEGUIMenuItem.Create(2,Fnt,SndClick,400*WndAspect,300*WndAspect,0.1,Language.Menu.Play));
  //GUI.AddCtrl(THGEGUIMenuItem.Create(3,Fnt,SndClick,400*WndAspect,340*WndAspect,0.2,Language.Menu.Options));
  GUI.AddCtrl(THGEGUIMenuItem.Create(4,Fnt,SndClick,400*WndAspect,340*WndAspect,0.3,Language.Menu.TOP));
  GUI.AddCtrl(THGEGUIMenuItem.Create(5,Fnt,SndClick,400*WndAspect,380*WndAspect,0.4,Language.Menu.Credits));
  GUI.AddCtrl(THGEGUIMenuItem.Create(6,Fnt,SndClick,400*WndAspect,420*WndAspect,0.5,Language.Menu.Exit));

  GUI.ShowCtrl(1,false);
  GUI.EnableCtrl(1,false);
  GUI.EnableCtrl(3,false);
  GUI.SetNavMode(HGEGUI_UPDOWN or HGEGUI_CYCLED);
  GUI.SetCursor(CursorSpr);
  GUI.SetFocus(2);
  GUI.Enter;
  LastId := 0;
  About := TAbout.Create;
  profile.Load;
end;

destructor TMenu.Destroy;
begin
  Profile.Save;
  about.Destroy;
  SndClick := nil;
  inherited;
end;

procedure TMenu.Activate;
begin
  Game.State := gsMenu;
  Game.Pause := true;
  Channel.Resume;
end;

procedure TMenu.DeActivate;
begin
  Game.Pause := false;
  Channel.pause;
  Game.State := gsGame;
end;

procedure TMenu.Draw;
begin
  //_HGE.Gfx_Clear($FF8800);
  Spr.RenderStretch(0,0,WND_WIDTH,WND_HEIGHT);
  case LastId of
    4 : Top.Render;
    5 : About.Render;
  else begin
    if Profile.bEnterName1 then ProfileEnterDownSpr.RenderEx(160*WndAspect,SCR_CENTER_H+50*WndAspect,0,WndAspect*1.5,1.5*WndAspect)
    else ProfileEnterUpSpr.RenderEx(160*WndAspect,SCR_CENTER_H+50*WndAspect,0,WndAspect*1.5,1.5*WndAspect);
    ProfileEnterMuchSpr1.RenderEx(160*WndAspect,SCR_CENTER_H+50*WndAspect-100*WndAspect,0,WndAspect*1.5,1.5*WndAspect);
    Fnt.SetColor(Game.GamePlayColorAll[0]);
    Fnt.PrintF(160*WndAspect,SCR_CENTER_H+50*WndAspect,HGETEXT_CENTER,Profile.PlayerName,[]);
    //BtnGameType.Draw;
    GUI.Render;
   end;
  end;
end;

procedure TMenu.Update(const delta : single);
var Id: INteger;
  X,Y : single;
begin
  _HGE.Input_GetMousePos(X,Y);
  
  if _HGE.Input_KeyDown(HGEK_LBUTTON) then begin
    {if BtnGameType.isActive then begin
      if GameType=ltTime then GameType:=ltWiner else GameType:=ltTime;
      BtnGameType.Anim.SetFrame(ord(TLevelType(GameType)));
      SndClick.Play;
    end;}
    if (X>(160-96)*WndAspect)and(X<(160+96)*WndAspect)and(Y>SCR_CENTER_H+50*WndAspect)and(Y<SCR_CENTER_H+50*WndAspect+(48)*WndAspect) then
       Profile.bEnterName1 := true else Profile.bEnterName1 := false;
  end;
  // We update the GUI and take an action if
  // one of the menu items was selected
  Profile.EnterName;
  if LastId>0 then
  if _HGE.Input_GetKeyState(HGEK_ESCAPE) then begin
    LastId := 0;
    GUI.SetFocus(LastId);
    if LevelManager.isPlaying then begin
      GUI.ShowCtrl(1,true);
      GUI.EnableCtrl(1,true);
    end;
    for id := 2 to ItemsCount do begin
               GUI.ShowCtrl(id,true);
               GUI.EnableCtrl(id,true);
    end;
    GUI.EnableCtrl(3,false);
    GUI.Enter;
  end;
  
  Id := GUI.Update(delta);
    if (Id = -1) then begin
      case LastId of
        1 : begin
          DeActivate;
          GUI.SetFocus(1);
          GUI.Enter;
          Game.Channel.Resume;
        end;
        2 : begin
          GUI.SetFocus(2);
          GUI.Enter;
          GUI.ShowCtrl(1,true);
          GUI.EnableCtrl(1,true);
          Profile.Save;
          DeActivate;
          LevelManager.NewGame;
        end;
        4..5: begin
             for id := 1 to ItemsCount do begin
               GUI.ShowCtrl(id,false);
               GUI.EnableCtrl(id,false);
             end;
             GUI.Enter;
           end;
        6: begin
           Game.Finish := true;
           GUI.Leave;
         end;
      end;
    end else if (Id <> 0) then begin
      LastId := Id;
      GUI.Leave;
    end;
  case LastId of
    4 : TOP.Update(delta);
    5 : About.Update(delta);
  end;
end;

end.
