unit g_monster_bee;
interface
uses
  Windows,Classes,
  HGE,HGESprite,HGEParticle,
  g_Types,g_cfg,g_math,g_Options;

type
  TBullet= class
    X,Y : Single;
    particle : IHGEParticleSystem;
    active : boolean;
  end;

  TMonster_BEE= class
    X,Y,Angle,tAngle : Single;
    TargetX,TargetY : single;
    particle : IHGEParticleSystem;
    Life : integer;
    List : TList;
    ReloadTime : single;
    constructor Create;
    procedure Init;
    function AddBullet : integer;
  end;

  TMonsterBEEManager = class
   public
     Tex : ITexture;
     Spr : IHGESprite;
     TexParticle : ITexture;
     List : TList;
     MonsterSpeed : single;
     MaxLife : integer;
     Radius : single;
     Speed : single;

     BulletSpeed : single;
     BulletTex : ITexture;
     BulletSpr : IHGESprite;
     ReloadDelay : single;
     BulletDamage : word;
     BulletRadius : single;
     SndDead : IEffect;
   public
    constructor Create;
    procedure Init;
    procedure Render();
    procedure Update(const delta : single);
    procedure AddMonster(const _X,_Y : single);
    procedure Damage(ID : word; Value : word);
    procedure Dead(id : word);
  end;

var
  Monsters_BEE : TMonsterBEEManager;
  Monster_Particle_Tex : ITexture;

implementation
uses
  g_Player;

constructor TMonster_BEE.Create;
begin
  inherited;
  List := TList.Create;

end;

procedure TMonster_BEE.Init;
begin
  List.Clear;
end;

function TMonster_BEE.AddBullet : integer;
var
  i : word;
begin
  Result := -1;
  i:=0;
  while i<List.Count do begin
    with TBullet(List.Items[i]) do begin
      if not active then begin
        Result := i;
        break;
      end;
    end;
    inc(i);
  end;

  if Result = -1 then Result := List.Add(TBullet.Create);
  TBullet(List.Items[Result]).X := X;
  TBullet(List.Items[Result]).Y := Y;
  TBullet(List.Items[Result]).active := true;
end;

constructor TMonsterBEEManager.Create;
var
  FileName : string;
begin
  inherited;
  FileName := 'data\cfg\monster_BEE.cfg';
  Tex := _HGE.Texture_Load(Cfg_ReadStr(FileName,'TEXTURE',''));
  Spr := THGESprite.Create(Tex,0,0,Tex.GetWidth,Tex.GetHeight);
  Spr.SetHotSpot(Tex.GetWidth / 2,Tex.GetHeight / 2);
  Spr.SetColor($ffffffff);
  Monster_Particle_Tex := _HGE.Texture_Load(Cfg_ReadStr(FileName,'TEXTURE_PARTICLE','particles.png'));
  List := TList.Create;
  MonsterSpeed := Cfg_ReadInt(FileName,'SPEED',6)*WndAspect/10;
  MaxLife := Cfg_ReadInt(FileName,'LIFE',6);
  Radius := MONSTER_RADIUS/2*WndAspect;

  BulletTex := _HGE.Texture_Load(Cfg_ReadStr(FileName,'BULLET_TEXTURE',''));
  BulletSpr := THGESprite.Create(BulletTex,0,0,BulletTex.GetWidth,BulletTex.GetHeight);
  BulletSpr.SetColor($ff00ff00);
  BulletSpr.SetHotSpot(BulletTex.GetWidth / 2,BulletTex.GetHeight / 2);
  BulletSpeed := Cfg_ReadInt(FileName,'BULLET_SPEED',6)*WndAspect;
  ReloadDelay := Cfg_ReadInt(FileName,'RELOAD_DELAY',6)/100;
  BulletDamage := Cfg_ReadInt(FileName,'BULLET_DAMAGE',6);
  BulletRadius := 16*WndAspect;
  sndDead := _HGE.Effect_Load(Cfg_ReadStr(FileName,'SOUND_DEAD','rocket_fire.wav'));
  Speed := 2;
end;

procedure TMonsterBEEManager.Init;
begin
  //sndFire := SoundEngine.LoadSound(dir_local+Cfg_ReadStr(FileName,'sndFire','data\snd\default.wav'));
  List.Clear;
end;

procedure TMonsterBEEManager.AddMonster(const _X,_Y : single);
var
  id : integer;
  i : word;
  j: Integer;
begin
  for j := 0 to 3 - 1 do begin
  id := -1;
  i:=0;
  while i<List.Count do begin
    with TMonster_BEE(List.Items[i]) do begin
      if Life < 0 then begin
        ID := i;
        break;
      end;
    end;
    inc(i);
  end;

  if id =-1 then id := List.Add(TMonster_BEE.Create);
  with TMonster_BEE(List.Items[id]) do begin
    X := _X+j*10;
    Y := _Y;
    TargetX := _X;
    TargetY := _Y;
    Life := MaxLife;
    ReloadTime := random(round(ReloadDelay*10))/10;
    Angle := 90;
    tAngle := 90;
  end;
  end;
end;

procedure TMonsterBEEManager.Damage(ID : word; Value : word);
begin
  dec(TMonster_BEE(List.Items[ID]).Life,Value);
  if TMonster_BEE(List.Items[ID]).Life<=0 then begin
    Dead(ID);
    Profile.tScore := Profile.tScore + 5;
  end;
end;

procedure TMonsterBEEManager.Dead(id : word);
begin
  //if TMonster(List.Items[id]).Particle <> nil then
    //TMonster(List.Items[id]).Particle.FireAt(round(TMonster(List.Items[id]).X),round(TMonster(List.Items[id]).Y));
  ToggleParSys(TMonster_BEE(List.Items[id]).Particle,'data\particles\particle_monster_dead.psi',Monster_Particle_Tex,64,0,32,
               round(TMonster_BEE(List.Items[id]).X),round(TMonster_BEE(List.Items[id]).Y),
               BLEND_ALPHABLEND or BLEND_COLORMUL or BLEND_ALPHAADD,$ffff8800,$ffffff00);
  TMonster_BEE(List.Items[id]).Life := -1;
  sndDead.PlayEx(Options.SoundVolume);
  inc(Player.DeadCount);
end;

procedure TMonsterBEEManager.Update(const delta : single);
var i,j : word;
begin
  i:=0;
  while i<List.Count do begin
    if TMonster_BEE(List.Items[i]).Life > 0 then
    with TMonster_BEE(List.Items[i]) do begin
      Angle := Angle + (tAngle-Angle)/5;
      if Distance2D(X,Y,TargetX,TargetY)<32 then begin
        TargetX := Random(WND_WIDTH);
        TargetY := Random(WND_HEIGHT);
        tAngle := GetAngle(X,Y,TargetX,TargetY);
      end;
      X := X + Speed*cos(deg2rad*Angle);
      Y := Y + Speed*sin(deg2rad*Angle);
      if (Y > WND_HEIGHT)or(Y < 0) or (X > WND_WIDTH)or(X < 0) then Life := -1;
      if Distance2D(X,Y,Player.X,Player.Y) < Radius+Player.Radius then begin
        Player.Damage(10);
        Dead(i);
      end;
      // Add bullet
        ReloadTime := ReloadTime + delta;
        if ReloadTime > ReloadDelay then begin
          AddBullet;
          ReloadTime := 0;
        end;
    end;
    j := 0;
      while j< TMonster_BEE(List.Items[i]).List.Count do begin
        with TBullet(TMonster_BEE(List.Items[i]).List.Items[j]) do
        if active then begin
          if (Y > WND_HEIGHT-10)or(Y < 10)
          or (X > WND_WIDTH-10)or(X < 10) then
            active := false else begin
              Y := Y + BulletSpeed;
              if Distance2D(Player.X,Player.Y,X,Y)< Player.Radius then begin
                active := false;
                Player.Damage(BulletDamage);
              end;
            end;
          end;
        inc(j);
      end;
    inc(i);
  end;
end;

procedure TMonsterBEEManager.Render();
var i,j : word;
begin
  i := 0;
  while i < List.Count do begin
    if TMonster_BEE(List.Items[i]).Life > 0 then
      Spr.RenderEx(TMonster_BEE(List.Items[i]).X,TMonster_BEE(List.Items[i]).Y,deg2rad*TMonster_BEE(List.Items[i]).angle,MONSTER_SCALE*WndAspect);
      j := 0;
      while j < TMonster_BEE(List.Items[i]).List.Count do begin
        if TBullet(TMonster_BEE(List.Items[i]).List.Items[j]).active then
        BulletSpr.RenderEx(TBullet(TMonster_BEE(List.Items[i]).List.Items[j]).X,TBullet(TMonster_BEE(List.Items[i]).List.Items[j]).Y,deg2rad*180,1*WndAspect);
        inc(j);
      end;
  inc(i);
  end;
end;

end.
