unit g_monster_eye;

interface
uses
  Windows,Classes,
  HGE,HGESprite,HGEParticle,g_Options,
  g_Types,g_cfg,g_math;

type
  TBullet= class
    X,Y : Single;
    particle : IHGEParticleSystem;
    active : boolean;
  end;

  TMonster= class
    X,Y : Single;
    particle : IHGEParticleSystem;
    Life : integer;
    List : TList;
    ReloadTime : single;
    Angle : single;
    constructor Create;
    procedure Init;
    function AddBullet : integer;
  end;

  TMonsterEYEManager = class
   public
     Tex : ITexture;
     Spr : IHGESprite;
     TexParticle : ITexture;
     List : TList;
     MonsterSpeed : single;
     MaxLife : integer;
     Radius : single;

     BulletSpeed : single;
     BulletTex : ITexture;
     BulletSpr : IHGESprite;
     ReloadDelay : single;
     BulletDamage : word;
     BulletRadius : single;
     sndDead : IEffect;
   public
    constructor Create;
    procedure Init;
    procedure Render();
    procedure Update(const delta : single);
    procedure AddMonster(const _X,_Y : single);
    procedure Damage(ID : word; Value : word);
    procedure Dead(id : word);
  end;

var
  Monsters_EYE : TMonsterEYEManager;
  Monster_Particle_Tex : ITexture;

implementation
uses
  g_Player;

constructor TMonster.Create;
begin
  inherited;
  List := TList.Create;
end;

procedure TMonster.Init;
begin
  List.Clear;
end;

function TMonster.AddBullet : integer;
var
  i : word;
begin
  Result := -1;
  i:=0;
  while i<List.Count do begin
    with TBullet(List.Items[i]) do begin
      if not active then begin
        Result := i;
        break;
      end;
    end;
    inc(i);
  end;

  if Result = -1 then Result := List.Add(TBullet.Create);
  TBullet(List.Items[Result]).X := X;
  TBullet(List.Items[Result]).Y := Y;
  TBullet(List.Items[Result]).active := true;
end;

constructor TMonsterEYEManager.Create;
var
  FileName : string;
begin
  inherited;
  FileName := 'data\cfg\monster_eye.cfg';
  Tex := _HGE.Texture_Load(Cfg_ReadStr(FileName,'TEXTURE',''));
  Spr := THGESprite.Create(Tex,0,0,Tex.GetWidth,Tex.GetHeight);
  Spr.SetHotSpot(Tex.GetWidth / 2,Tex.GetHeight / 2);
  Spr.SetColor($ffffffff);
  Monster_Particle_Tex := _HGE.Texture_Load(Cfg_ReadStr(FileName,'TEXTURE_PARTICLE','particles.png'));
  List := TList.Create;
  MonsterSpeed := Cfg_ReadInt(FileName,'SPEED',6)*WndAspect/10;
  MaxLife := Cfg_ReadInt(FileName,'LIFE',6);
  Radius := MONSTER_RADIUS*WndAspect;

  BulletTex := _HGE.Texture_Load(Cfg_ReadStr(FileName,'BULLET_TEXTURE',''));
  BulletSpr := THGESprite.Create(BulletTex,0,0,BulletTex.GetWidth,BulletTex.GetHeight);
  BulletSpr.SetHotSpot(BulletTex.GetWidth / 2,BulletTex.GetHeight / 2);
  BulletSpr.SetColor($ffff0000);
  BulletSpeed := Cfg_ReadInt(FileName,'BULLET_SPEED',6)*WndAspect;
  ReloadDelay := Cfg_ReadInt(FileName,'RELOAD_DELAY',6)/100;
  BulletDamage := Cfg_ReadInt(FileName,'BULLET_DAMAGE',6);
  BulletRadius := 16*WndAspect;
  sndDead := _HGE.Effect_Load(Cfg_ReadStr(FileName,'SOUND_DEAD','rocket_fire.wav'));
end;

procedure TMonsterEYEManager.Init;
begin
  //sndFire := SoundEngine.LoadSound(dir_local+Cfg_ReadStr(FileName,'sndFire','data\snd\default.wav'));
  List.Clear;
end;

procedure TMonsterEYEManager.AddMonster(const _X,_Y : single);
var
 id : integer;
 i : word;
begin
  id := -1;
  i:=0;
  while i<List.Count do begin
    with TMonster(List.Items[i]) do begin
      if Life < 0 then begin
        ID := i;
        break;
      end;
    end;
    inc(i);
  end;

  if id =-1 then id := List.Add(TMonster.Create);
  with TMonster(List.Items[id]) do begin
    X := _X;
    Y := _Y;
    Life := MaxLife;
    ReloadTime := random(round(ReloadDelay*10))/10;
  end;
end;

procedure TMonsterEYEManager.Damage(ID : word; Value : word);
begin
  dec(TMonster(List.Items[ID]).Life,Value);
  if TMonster(List.Items[ID]).Life<=0 then begin
    Dead(ID);
    Profile.tScore := Profile.tScore + 25;
  end;
end;

procedure TMonsterEYEManager.Dead(id : word);
begin
  //if TMonster(List.Items[id]).Particle <> nil then
    //TMonster(List.Items[id]).Particle.FireAt(round(TMonster(List.Items[id]).X),round(TMonster(List.Items[id]).Y));
  ToggleParSys(TMonster(List.Items[id]).Particle,'data\particles\particle_monster_dead.psi',Monster_Particle_Tex,64,0,32,
               round(TMonster(List.Items[id]).X),round(TMonster(List.Items[id]).Y),
               BLEND_ALPHABLEND or BLEND_COLORMUL or BLEND_ALPHAADD,$ffff0000,$ffffffff);
  TMonster(List.Items[id]).Life := -1;
  sndDead.PlayEx(Options.SoundVolume);
  inc(Player.DeadCount);
end;

procedure TMonsterEYEManager.Update(const delta : single);
var i,j : word;
begin
  i:=0;
  while i<List.Count do begin
    if TMonster(List.Items[i]).Life > 0 then
    with TMonster(List.Items[i]) do begin
      Y := Y + MonsterSpeed;
      if Y > WND_HEIGHT-10 then Life := -1;
      if Distance2D(X,Y,Player.X,Player.Y) < Radius+Player.Radius then begin
        Player.Damage(20);
        Dead(i);
      end;
      // Add bullet
        ReloadTime := ReloadTime + delta;
        if ReloadTime > ReloadDelay then begin
          AddBullet;
          ReloadTime := 0;
        end;
    end;
    j := 0;
      while j< TMonster(List.Items[i]).List.Count do begin
        with TBullet(TMonster(List.Items[i]).List.Items[j]) do
        if active then begin
          if (Y > WND_HEIGHT)or(Y < 0) or (X > WND_WIDTH)or(X < 0) then
            active := false else begin
            Y := Y + BulletSpeed;
            if Distance2D(Player.X,Player.Y,X,Y)< Player.Radius then begin
              active := false;
              Player.Damage(BulletDamage);
            end;
          end;
        end;
        inc(j);
      end;
    inc(i);
  end;
end;

procedure TMonsterEYEManager.Render();
var i,j : word;
begin
  i := 0;
  while i < List.Count do begin
    if TMonster(List.Items[i]).Life > 0 then
      Spr.RenderEx(TMonster(List.Items[i]).X,TMonster(List.Items[i]).Y,deg2rad*180,MONSTER_SCALE*WndAspect);
      j := 0;
      while j < TMonster(List.Items[i]).List.Count do begin
        if TBullet(TMonster(List.Items[i]).List.Items[j]).active then
        BulletSpr.RenderEx(TBullet(TMonster(List.Items[i]).List.Items[j]).X,TBullet(TMonster(List.Items[i]).List.Items[j]).Y,deg2rad*180,1*WndAspect);
        inc(j);
      end;
  inc(i);
  end;
end;

end.
