unit g_monster_fly;

interface
uses
  Windows,Classes,
  HGE,HGESprite,HGEParticle,
  g_Types,g_cfg,g_math,g_Options;

type
  TMonster_FLY = class
    X,Y : Single;
    particle : IHGEParticleSystem;
    Life : integer;
    constructor Create;
  end;

  TMonsterFLYManager = class
   public
     Tex : ITexture;
     Spr : IHGESprite;
     TexParticle : ITexture;
     List : TList;
     MonsterSpeed : single;
     MaxLife : integer;
     Radius : single;
     sndDead : IEffect;
   public
    constructor Create;
    procedure Init;
    procedure Render();
    procedure Update(const delta : single);
    procedure AddMonster(const _X,_Y : single);
    procedure Damage(ID : word; Value : word);
    procedure Dead(id : word);
  end;

var
  Monsters_FLY : TMonsterFLYManager;
  Monster_Particle_Tex : ITexture;

implementation
uses
  g_Player;

constructor TMonster_FLY.Create;
begin
  inherited;

end;

constructor TMonsterFLYManager.Create;
var
  FileName : string;
begin
  inherited;
  FileName := 'data\cfg\monster_fly.cfg';
  Tex := _HGE.Texture_Load(Cfg_ReadStr(FileName,'TEXTURE',''));
  Spr := THGESprite.Create(Tex,0,0,Tex.GetWidth,Tex.GetHeight);
  Spr.SetHotSpot(Tex.GetWidth / 2,Tex.GetHeight / 2);
  Monster_Particle_Tex := _HGE.Texture_Load(Cfg_ReadStr(FileName,'TEXTURE_PARTICLE','particles.png'));
  MonsterSpeed := Cfg_ReadInt(FileName,'SPEED',6)*WndAspect/10;
  MaxLife := Cfg_ReadInt(FileName,'LIFE',6);
  sndDead := _HGE.Effect_Load(Cfg_ReadStr(FileName,'SOUND_DEAD','rocket_fire.wav'));
  List := TList.Create;
end;

procedure TMonsterFLYManager.Init;
begin
  //sndFire := SoundEngine.LoadSound(dir_local+Cfg_ReadStr(FileName,'sndFire','data\snd\default.wav'));
  List.Clear;
end;

procedure TMonsterFLYManager.AddMonster(const _X,_Y : single);
var
 id : integer;
 i : word;
begin
  id := -1;
  i:=0;
  while i<List.Count do begin
    with TMonster_FLY(List.Items[i]) do begin
      if Life < 0 then begin
        ID := i;
        break;
      end;
    end;
    inc(i);
  end;

  if id =-1 then id := List.Add(TMonster_FLY.Create);
  with TMonster_FLY(List.Items[id]) do begin
    X := _X;
    Y := _Y;
    Life := MaxLife;
    Radius := MONSTER_RADIUS*WndAspect;
  end;
end;

procedure TMonsterFLYManager.Damage(ID : word; Value : word);
begin
  dec(TMonster_FLY(List.Items[ID]).Life,Value);
  if TMonster_FLY(List.Items[ID]).Life<=0 then begin
    Dead(ID);
    Profile.tScore := Profile.tScore + 10;
  end;
end;

procedure TMonsterFLYManager.Dead(id : word);
begin
  //if TMonster(List.Items[id]).Particle <> nil then
    //TMonster(List.Items[id]).Particle.FireAt(round(TMonster(List.Items[id]).X),round(TMonster(List.Items[id]).Y));
   ToggleParSys(TMonster_FLY(List.Items[id]).Particle,'data\particles\particle_monster_dead.psi',Monster_Particle_Tex,64,0,32,
               round(TMonster_FLY(List.Items[id]).X),round(TMonster_FLY(List.Items[id]).Y),
               BLEND_ALPHABLEND or BLEND_COLORMUL or BLEND_ALPHAADD,$ff00ff00,$ffffffff);
  TMonster_FLY(List.Items[id]).Life := -1;
  sndDead.PlayEx(Options.SoundVolume);
  inc(Player.DeadCount);
end;

procedure TMonsterFLYManager.Update(const delta : single);
var i : word;
begin
  i:=0;
  while i<List.Count do begin
    if TMonster_FLY(List.Items[i]).Life > 0 then
    with TMonster_FLY(List.Items[i]) do begin
      Y := Y + MonsterSpeed;
      if Y > WND_HEIGHT then Life := -1;
      if Distance2D(X,Y,Player.X,Player.Y) < Radius+Player.Radius then begin
        Player.Damage(20);
        Dead(i);
      end;
    end;
    inc(i);
  end;
end;

procedure TMonsterFLYManager.Render();
var i : word;
begin
  i := 0;
  while i < List.Count do begin
    if TMonster_FLY(List.Items[i]).Life > 0 then
    with TMonster_FLY(List.Items[i]) do begin
      Spr.SetColor($ffffffff);
      Spr.RenderEx(X,Y,deg2rad*180,MONSTER_SCALE*WndAspect);
    end;
  inc(i);
  end;
end;

end.
