unit g_player;

interface
uses
  HGE,HGEFont,HGESprite,HGEParticle,
  g_Types,g_cfg,math,g_math,g_Options,
  g_gun_mini,g_gun_rocket,g_gun_voice;

type
  TShinigami = class
    Active : boolean;                                                          //  
    Armor : single;                                                            //   
    Tex : ITexture;
    Spr :IHGESprite;
    Particle : IHGEParticleSystem;
    Color : Cardinal;
    MaxCount,Count : single;                                                   //  ""
    Time,Delay : single;                                                       //  
    constructor Create;
    procedure Update(delta: Single;D_Count : word);
    procedure Render(_X,_Y : single);
  end;

  TPlayer = class
  public
    tex : ITexture;
    spr : IHGESprite;
    TexParticle : ITexture;
    X,Y : single;
    Size,Radius : single;
    MaxSpeed,SpeedX,SpeedY,
    AccelerationX,AccelerationY : single;
    Life : integer;

    MaxLevel,Level : byte;
    LevelUp : array [1..6] of cardinal;

    bLeft, bRight, bForward, bBackward : boolean;

    GunMini : TGunMini;
    GunRocket : TGunRocket;
    GunVoice : TGunVoice;
    DeadCount : word;

    particle_L,particle_R : IHGEParticleSystem;
    sndLevelUp : IEffect;
    Shinigami : TShinigami;
  public
    constructor create;
    destructor Destroy;
    procedure Init;
    procedure Update(delta:single);
    procedure Render;
    procedure Left;
    procedure Right;
    procedure pressGas;
    procedure pressBreak;
    procedure Damage(Value : word);
    procedure Dead;
  end;


var
  Player : TPlayer;

implementation

constructor TShinigami.create;
begin
  inherited;
  Active := false;
  Armor := 0.25;
  Tex := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\player.cfg','TEXTURE_SHINIGAMI','shinigami.png'));
  Spr := THGESprite.Create(Tex,0,0,Tex.GetWidth,Tex.GetHeight);
  Spr.SetHotSpot(Tex.GetWidth / 2,Tex.GetHeight / 2);
  Spr.SetColor($ffffffff);
  //Particle : IHGEParticleSystem;
  Color := $ff00ff00;
  MaxCount := 5;
  Count := 5;
  Time := 0;
  Delay := 10;
end;

procedure TShinigami.Render(_X,_Y : single);
begin
  Spr.RenderEx(_X,_Y,0,0.5*WndAspect);
end;

procedure TShinigami.Update(delta: Single;D_Count : word);
begin
  {if D_Count>=Count then begin
    Active := true;
    Count := Count + MaxCount;
  end; }
end;

constructor TPlayer.create;
begin
  inherited;
  Tex := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\player.cfg','TEXTURE','player.png'));
  Spr := THGESprite.Create(Tex,0,0,Tex.GetWidth,Tex.GetHeight);
  Spr.SetHotSpot(Tex.GetWidth / 2,Tex.GetHeight / 2);
  Spr.SetColor($ffffffff);
  TexParticle := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\player.cfg','TEXTURE_PARTICLE','particles.png'));
  X := SCR_CENTER_W;
  Y := WND_HEIGHT-SCR_CENTER_H div 4;
  Size := 64*WndAspect;
  Radius := Size/2;
  MaxSpeed := cfg_ReadInt('data\cfg\player.cfg','MAX_SPEED',7)*WndAspect;
  AccelerationX := cfg_ReadInt('data\cfg\player.cfg','ACCELERATION',20)/10*WndAspect;
  AccelerationY := AccelerationX/2;
  SpeedX := 0;
  SpeedY := 0;
  GunMini := TGunMini.Create;
  GunRocket := TGunRocket.Create;
  GunVoice := TGunVoice.Create;
  DeadCount := 0;
  MaxLevel := 6;
  Life := 100;
  LevelUp[1] := 350;
  LevelUp[2] := 900;
  LevelUp[3] := 2000;
  LevelUp[4] := 4000;
  LevelUp[5] := 6000;
  LevelUp[6] := 8000;
  sndLevelUp := _HGE.Effect_Load(Cfg_ReadStr('data\cfg\player.cfg','SOUND_LEVELUP','levelup.wav'));
  Shinigami := TShinigami.Create;
end;

destructor TPlayer.Destroy;
begin
  GunMini.free;
  GunRocket.free;
  GunVoice.free;
  sndLevelUp := nil;
  inherited;
end;

procedure TPlayer.Init;
begin
  bLeft := false;
  bRight := false;
  bForward := false;
  bBackward := false;
  X := SCR_CENTER_W;
  Y := WND_HEIGHT-SCR_CENTER_H div 4;
  ToggleParSys(Particle_L,'data\particles\particle_gun.psi',TexParticle,96,64,32,round(X),round(Y),BLEND_ALPHAADD,$aa33aaee);
  ToggleParSys(Particle_R,'data\particles\particle_gun.psi',TexParticle,96,64,32,round(X),round(Y),BLEND_ALPHAADD,$aa33aaee);
  Life := 100;
  Level := 1;
  GunMini.Init;
  GunRocket.Init;
  GunVoice.Init;
end;

procedure TPlayer.update(delta:single);
begin
  if Level > 4 then Shinigami.Active := true;
  
  Shinigami.Update(delta,DeadCount);
  if (bForward) then begin // 
    if SpeedY > -MaxSpeed then SpeedY := SpeedY - AccelerationY;
  end else
  if (bBackward) then begin
    if SpeedY < MaxSpeed then SpeedY := SpeedY + AccelerationY;
  end;

  if (bRight) then begin
    if SpeedX < MaxSpeed then SpeedX := SpeedX + AccelerationX;
  end else
  if (bLeft) then begin
    if SpeedX > -MaxSpeed then SpeedX := SpeedX - AccelerationX;
  end;

  Aspire(SpeedX,0,abs(SpeedX)*0.1);
  Aspire(SpeedY,0,abs(SpeedY)*0.1);
  X := X + SpeedX;
  Y := Y + SpeedY;
  //  
  if X+Radius > WND_WIDTH then X := WND_WIDTH-Radius else
   if X-Radius < 0 then X := Radius;
  if Y+Radius > WND_HEIGHT then Y := WND_HEIGHT-Radius else
   if Y-Radius < 0 then Y := Radius;
//  
  bLeft := false;
  bRight := false;
  bForward := false;
  bBackward := false;

  if particle_L<>nil then particle_L.MoveTo(X-25*WndAspect,Y);
  if particle_R<>nil then particle_R.MoveTo(X+25*WndAspect,Y);
  GunMini.Update(delta,X,Y);
  GunRocket.Update(delta,X,Y);
  GunVoice.Update(delta,X,Y);

  //Level Up
  if LevelUp[Level] <= Profile.tScore then
  if Player.level < Player.MaxLevel then begin
    inc(Player.level);
    sndLevelUp.PlayEx(Options.SoundVolume);
  end;
end;

procedure TPlayer.Render;
begin
  GunMini.Render;
  GunRocket.Render;
  GunVoice.Render;
  if Shinigami.Active then Shinigami.Render(X,Y)
   else Spr.RenderEx(X,Y,0,0.5*WndAspect);
end;

procedure TPlayer.Left;
begin
  bLeft := true;
  //particle_L.Fire();
  //particle_R.Stop;
end;

procedure TPlayer.Right;
begin
  bRight := true;
  //particle_L.Stop;
  //particle_R.Fire;
end;

procedure TPlayer.pressGas;
begin
  bForward := true;
end;

procedure TPlayer.pressBreak;
begin
  bBackward := true;
end;

procedure TPlayer.Damage(Value : word);
begin
  Life := Life - Value;
  if Life <= 0 then Dead;  
end;

procedure TPlayer.Dead;
begin
  LevelManager.GameOver;
end;

end.
